import subprocess, os, os.path, sys, re, shutil

#OBJDUMP = '/opt/mxe/usr/bin/i686-w64-mingw32.shared-objdump'
#BASEDIR = '/opt/mxe/usr/i686-w64-mingw32.shared/bin/'
OBJDUMP = 'objdump'
BASEDIR = 'c:/msys64/mingw64/bin'

def getDependencies(filename):
	out = subprocess.check_output([OBJDUMP, '-x', filename], universal_newlines = True)
	return set(re.findall('DLL Name: (.*)', out))

def getDependenciesRecursively(filename, checkedAlready = None):
	rv = checkedAlready if checkedAlready is not None else set()
	for dep in getDependencies(filename):
		if dep in rv: continue

		rv.add(dep)
		filename = os.path.join(BASEDIR, dep)
		if not os.path.exists(filename): continue

		rv = rv.union(getDependenciesRecursively(filename, rv))
	return rv



if len(sys.argv) != 2:
	print('Usage: %s file.exe' % sys.argv[0])
	sys.exit(1)

print('Checking dependencies for %s' % sys.argv[1])

OUTDIR = os.path.dirname(sys.argv[1])
# Replace an empty OUTDIR by '.' to have a nicer output
if OUTDIR == '': OUTDIR = '.'

for dep in sorted(getDependenciesRecursively(sys.argv[1])):
	src = os.path.normpath(os.path.join(BASEDIR, dep))
	dst = os.path.normpath(os.path.join(OUTDIR, dep))

	if os.path.exists(dst):
		print('Skipping %s - already in %s' % (dep, OUTDIR))
		continue
	if not os.path.exists(src):
		print('Skipping %s - not in %s' % (dep, BASEDIR))
		continue
	print('%s > %s' % (src, dst))
	shutil.copy(src, dst)



import json, os, os.path, zipfile, tempfile
try:
	# Python3
	from urllib.request import build_opener
except ImportError:
	# Python2
	from urllib2 import build_opener

# Use urllib's build_opener() to define urlopen and urlretrieve as bandit
# (https://github.com/PyCQA/bandit) does not deem that to be as bad as the
# built-in variants

url_opener = build_opener()

def urlopen(url):
	return url_opener.open(url)

def urlretrieve(url, filename):
	fin = urlopen(url)
	with open(filename, 'wb') as fout:
		fout.write(fin.read())
	fin.close()


fin = urlopen("https://api.github.com/repos/TeXworks/manual/releases/latest")
release = json.loads(fin.read().decode('utf-8'))
fin.close()

for asset in release["assets"]:
	if asset['content_type'] == "application/zip":
		folder = "texworks-help"
		if not os.path.exists(folder):
			os.makedirs(folder)
		print("Downloading %s from %s and uncompressing to %s" % (asset['name'], asset['browser_download_url'], folder))
		tmpfile = tempfile.NamedTemporaryFile(delete = False)
		tmpfile.close()
		urlretrieve(asset['browser_download_url'], tmpfile.name)

		with zipfile.ZipFile(tmpfile.name, 'r') as z:
			z.extractall(folder)
		os.remove(tmpfile.name)

	elif asset['content_type'] == "application/pdf" or asset['content_type'] == "[application/pdf]":
		lang = os.path.splitext(asset['name'])[0].rsplit('-', 1)[1]
		folder = os.path.join("texworks-help", "TeXworks-manual", lang)
		if not os.path.exists(folder):
			os.makedirs(folder)
		print("Downloading %s from %s to %s" % (asset['name'], asset['browser_download_url'], folder))
		urlretrieve(asset['browser_download_url'], os.path.join(folder, asset['name']))

	else:
		print("Error: Unknown content type '%s' for file '%s'" % (asset['content_type'], asset['name']))



#!/usr/bin/env python3
#
# System requirements:
#  - GitPython: https://pypi.python.org/pypi/GitPython/
#
from __future__ import print_function
import git
from git import Repo
import datetime, os, re

# Global variable for the git repository
gitrepo = None

class FileNotVersionedError(Exception):
	def __init__(self, filename):
		super(FileNotVersionedError, self).__init__("File '%s' is not versioned" % filename)

class CopyrightedFile:
    """A thin wrapper for a real file on the filesystem. This class assists
    in updating the file's embedded copyright information.

    Attributes
    ----------

    RE_PART_OF_TEXWORKS: string
                         A regular expression to match the embedded copyright
                         line in files.
    REPLACE_EXTENSIONS: list[string]
                        Files ending in any of these extensions will be
                        considered when updating copyrighted files.

    """

    RE_PART_OF_TEXWORKS = "(\s*Copyright \(C\)) [-0-9]+  ([^\n]+)"
    REPLACE_EXTENSIONS = ['.cpp', '.h']
    EXCLUDE_FOLDERS = ['./modules/synctex/']

    def __init__(self, filename):
        """Construct from filename.

        Parameters
        ----------
        filename: str
                  Path to a file. This file's copyright information will be
                  examined and possibly updated by using the helper methods
                  below.

        """
        self.filename = filename
        self.log = None

    @property
    def year_range_str(self):
        """Returns the year (range) in which this file was modified"""
        global gitrepo
        timestamp_str = gitrepo.log('--diff-filter=A', '--follow', r'--format="%at"', '--', self.filename)
        if len(timestamp_str.strip()) == 0: raise FileNotVersionedError(self.filename)
        timestamp = float(timestamp_str.replace('"', ''))
        begin = datetime.datetime.fromtimestamp(timestamp).year
        logs = gitrepo.log('--format="%h %at %s"', '--', self.filename).split('\n')
        for log in logs:
            log = log[1:-1]
            commit, timestamp_str, msg = log.split(' ', 2)
            if msg in ['Update copyrights', 'Update copyright notices', 'Update copyright statements', 'Updated copyright information', 'Update copyright and add Charlie to list of authors', 'Update URLs to http://www.tug.org/texworks/', 'update copyright to 2010']: continue
            self.log = (commit, timestamp_str, msg)
            break
#        print(logs)
#        timestamp_str = gitrepo.log('-1', '--format="%at"', '--', self.filename)
        timestamp = float(timestamp_str.replace('"', ''))
        end = datetime.datetime.fromtimestamp(timestamp).year
        if begin == end:
            return str(end)
        else:
            return "{0}-{1}".format(begin, end)


    def one_of_replace_extensions(self):
        """Returns True if the file extension is allowed to be updated, False otherwise."""
        for ext in self.REPLACE_EXTENSIONS:
            if self.filename.endswith(ext):
                return True

        return False

    def is_excluded(self):
        for d in self.EXCLUDE_FOLDERS:
            if self.filename.startswith(d):
                return True
        return False

    def is_updateable(self):
        return not self.is_excluded() and self.one_of_replace_extensions()

    def needs_update(self):
        """Returns True if the file needs updating, False otherwise."""

        self.content = open(self.filename).read()
        self.matches = re.search(self.RE_PART_OF_TEXWORKS, self.content[:1000])
        if self.matches is None:
            print('    [WARNING] No Copyright notice found in {0}'.format(self.filename))
        return self.matches

    def update_copyright(self):
        """Updates the copyright line in the file."""

        # Replace the year(s)
        orig = self.matches.group(0)
        try:
	        subst = "{0} {1}  {2}".format(self.matches.group(1), self.year_range_str, self.matches.group(2))
        except FileNotVersionedError:
            return

        if subst == orig:
            print('   Already up to date')
            return
        self.content = self.content.replace(orig, subst)

        # Write the contents to disk
        f = open(self.filename, 'w')
        f.write(self.content)
        f.close()

        timestamp = datetime.datetime.fromtimestamp(float(self.log[1].replace('"', ''))).strftime('%c')

        print("   Last commit: {0} ({1}) {2}".format(self.log[0], timestamp, self.log[2]))
        print("   Updated {0}".format(self.filename))

def manual_update_notice():
    """Reminder for places where the copyright information must be updated manually"""
    print("")
    print("Don't forget to manually update the copyright information in the following files:")
    for f in ["README.md", "texworks.appdata.xml", "TeXworks.plist.in", "CMake/Modules/COPYING-CMAKE-MODULES", "man/texworks.1", "res/TeXworks.rc", "src/main.cpp", "src/TWApp.cpp", "ci/travis-ci/launchpad/debian/copyright", "ci/travis-ci/README.win"]:
    	print("   {0}".format(f))

def main():
    """Main"""
    global gitrepo
    gitrepo = Repo(".").git

    for f in git.cmd.Git().ls_files().split():
        the_file = CopyrightedFile(f)
        if not the_file.is_updateable(): continue
        print(f)
        if the_file.needs_update():
            the_file.update_copyright()
        else:
            print('    Skipping')

    manual_update_notice()

if __name__ == '__main__':
    main()
